﻿using System;
using System.ComponentModel.Composition;
using Microsoft.Office.Interop.Word;

namespace Songhay.Office2010.WordWalkingStick.Commands
{
    using Songhay.Models;
    using Songhay.Office2010.Word;

    [Export(typeof(IOfficeWordCommand))]
    public class OfficeWordOptionCommand : IOfficeWordCommand
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="OfficeWordOptionCommand"/> class.
        /// </summary>
        public OfficeWordOptionCommand()
        {
            this.Name = "OfficeWordOptionCommand";
        }

        #region IOfficeWordCommand Members

        /// <summary>
        /// Gets or sets the name
        /// (non-unique key)
        /// for locating this command
        /// in a collection of commands.
        /// </summary>
        /// <value>The name.</value>
        /// <remarks>
        /// Conventional command name prefixes:
        /// Startup* Commands with this prefix will be executed during the ThisAddIn_Startup event.
        /// Shutdown* Commands with this prefix will be executed during the ThisAddIn_Shutdown event.
        /// </remarks>
        public string Name { get; set; }

        /// <summary>
        /// Executes the command.
        /// </summary>
        /// <typeparam name="TMessage">The type of the message.</typeparam>
        /// <param name="wordApplication">The word application.</param>
        /// <param name="message">The message.</param>
        public void ExecuteCommand<TMessage>(Application wordApplication, TMessage message) where TMessage : IControlInteractionMessage
        {
            var m = message as IControlInteractionMessage;
            if(m == null) return;
            var option = (WordOptions)Enum.Parse(typeof(WordOptions), m.ControlId);
            switch(option)
            {
                case WordOptions.AllOptions:
                    wordApplication.Options.AllowOpenInDraftView = true;
                    break;

                case WordOptions.AllowOpenInDraftView:
                    wordApplication.Options.AllowOpenInDraftView = (bool)m.ControlValue;
                    break;

                case WordOptions.FormsDesign:
                    //TODO: consider verifying that Word starts with FormsDesign == false.
                    wordApplication.ActiveDocument.ToggleFormsDesign();
                    break;
            }
        }

        /// <summary>
        /// Executes the command for binding.
        /// </summary>
        /// <typeparam name="TMessage">The type of the message.</typeparam>
        /// <param name="wordApplication">The word application.</param>
        /// <param name="message">The message.</param>
        public object ExecuteCommandForBinding<TMessage>(Application wordApplication, TMessage message) where TMessage : IControlInteractionMessage
        {
            var m = message as IControlInteractionMessage;
            if(m == null) return null;
            var option = (WordOptions)Enum.Parse(typeof(WordOptions), m.ControlId);

            switch(option)
            {
                case WordOptions.AllowOpenInDraftView:
                    return wordApplication.Options.AllowOpenInDraftView;
                case WordOptions.FormsDesign:
                    return wordApplication.ActiveDocument.FormsDesign;
            }

            return null;
        }

        #endregion
    }
}
